@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2023 The SABnzbd-Team (sabnzbd.org)
+# Copyright 2007-2024 by The SABnzbd-Team (sabnzbd.org)
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -20,9 +20,11 @@ sabnzbd.misc - misc classes
 """
 
 import os
+import platform
 import ssl
 import sys
 import logging
+import functools
 import urllib.request
 import urllib.parse
 import re
@@ -31,12 +33,13 @@ import socket
 import time
 import datetime
 import inspect
+import queue
 import ctypes
 import html
 import ipaddress
 import socks
 from threading import Thread
-from typing import Union, Tuple, Any, AnyStr, Optional, List, Dict
+from typing import Union, Tuple, Any, AnyStr, Optional, List, Dict, Collection
 
 import sabnzbd
 import sabnzbd.getipaddress
@@ -50,11 +53,13 @@ from sabnzbd.constants import (
 )
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
-from sabnzbd.encoding import ubtou
-from sabnzbd.filesystem import userxbit, make_script_path, remove_file, is_valid_script
+from sabnzbd.decorators import cache_maintainer
+from sabnzbd.encoding import ubtou, platform_btou
+from sabnzbd.filesystem import userxbit, make_script_path, remove_file
 
 if sabnzbd.WIN32:
     try:
+        import winreg
         import win32process
         import win32con
 
@@ -82,9 +87,17 @@ RE_IP6 = re.compile(r"inet6\s+(addr:\s*)?([0-9a-f:]+)", re.I)
 HAVE_AMPM = bool(time.strftime("%p"))
 
 
-def helpful_warning(*args, **kwargs):
+def helpful_warning(msg, *args, **kwargs):
     """Wrapper to ignore helpful warnings if desired"""
-    if sabnzbd.cfg.helpful_warnings():
+    if cfg.helpful_warnings():
+        msg = "%s\n%s" % (msg, T("To prevent all helpful warnings, disable Special setting 'helpful_warnings'."))
+        return logging.warning(msg, *args, **kwargs)
+    return logging.info(msg, *args, **kwargs)
+
+
+def duplicate_warning(*args, **kwargs):
+    """Wrapper to ignore duplicate warnings if desired"""
+    if cfg.warn_dupl_jobs():
         return logging.warning(*args, **kwargs)
     return logging.info(*args, **kwargs)
 
@@ -159,6 +172,11 @@ def safe_lower(txt: Any) -> str:
         return ""
 
 
+def is_none(inp: Any) -> bool:
+    """Check for 'not X' but also if it's maybe the string 'None'"""
+    return not inp or (isinstance(inp, str) and inp.lower() == "none")
+
+
 def cmp(x, y):
     """
     Replacement for built-in function cmp that was removed in Python 3
@@ -171,6 +189,37 @@ def cmp(x, y):
     return (x > y) - (x < y)
 
 
+class MultiAddQueue(queue.Queue):
+    def put_multiple(self, multiple_items: Collection):
+        """Take advantage of the dequeue used by Queue that has a very
+        fast extend method to add multiple items at once.
+        See: https://github.com/sabnzbd/sabnzbd/discussions/2704"""
+        with self.not_full:
+            self.queue.extend(multiple_items)
+            self.unfinished_tasks += len(multiple_items)
+            self.not_empty.notify()
+
+
+def cat_pp_script_sanitizer(
+    cat: Optional[str] = None,
+    pp: Optional[Union[int, str]] = None,
+    script: Optional[str] = None,
+) -> Tuple[Optional[Union[int, str]], Optional[str], Optional[str]]:
+    """Basic sanitizer from outside input to a bit more predictable values"""
+    # Cannot use "not pp" because pp can also be 0
+    if pp in ("", "-1"):
+        pp = None
+
+    # Check for valid script is performed in NzbObject init
+    if not script or script.lower() == "default":
+        script = None
+
+    if not cat or cat.lower() in ("default", "*"):
+        cat = None
+
+    return cat, pp, script
+
+
 def name_to_cat(fname, cat=None):
     """Retrieve category from file name, but only if "cat" is None."""
     if cat is None and fname.startswith("{{"):
@@ -216,7 +265,7 @@ def cat_to_opts(cat, pp=None, script=None, priority=None) -> Tuple[str, int, str
     return cat, pp, script, priority
 
 
-def pp_to_opts(pp: int) -> Tuple[bool, bool, bool]:
+def pp_to_opts(pp: Optional[int]) -> Tuple[bool, bool, bool]:
     """Convert numeric processing options to (repair, unpack, delete)"""
     # Convert the pp to an int
     pp = sabnzbd.interface.int_conv(pp)
@@ -296,7 +345,7 @@ def cat_convert(cat):
     If no match found, but the indexer-cat starts with the user-cat, return user-cat
     If no match found, return None
     """
-    if cat and cat.lower() != "none":
+    if not is_none(cat):
         cats = config.get_ordered_categories()
         raw_cats = config.get_categories()
         for ucat in cats:
@@ -336,8 +385,6 @@ _SERVICE_PARM = "CommandLine"
 
 def get_serv_parms(service):
     """Get the service command line parameters from Registry"""
-    import winreg
-
     service_parms = []
     try:
         key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, _SERVICE_KEY + service)
@@ -357,8 +404,6 @@ def get_serv_parms(service):
 
 def set_serv_parms(service, args):
     """Set the service command line parameters in Registry"""
-    import winreg
-
     serv = []
     for arg in args:
         serv.append(arg[0])
@@ -694,6 +739,51 @@ def get_macos_memory():
     return float(system_output.split()[1])
 
 
+@cache_maintainer(clear_time=3600)
+@functools.lru_cache(maxsize=None)
+def get_cpu_name():
+    """Find the CPU name (which needs a different method per OS), and return it
+    If none found, return platform.platform()"""
+
+    cputype = None
+
+    try:
+        if sabnzbd.WIN32:
+            key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, r"Hardware\Description\System\CentralProcessor\0")
+            cputype = winreg.QueryValueEx(key, "ProcessorNameString")[0]
+            winreg.CloseKey(key)
+
+        elif sabnzbd.MACOS:
+            cputype = subprocess.check_output(["sysctl", "-n", "machdep.cpu.brand_string"]).strip()
+
+        else:
+            with open("/proc/cpuinfo") as fp:
+                for myline in fp.readlines():
+                    if myline.startswith("model name"):
+                        # Typical line:
+                        # model name      : Intel(R) Xeon(R) CPU           E5335  @ 2.00GHz
+                        cputype = myline.split(":", 1)[1]  # get everything after the first ":"
+                        break  # we're done
+        cputype = platform_btou(cputype)
+    except:
+        # An exception, maybe due to a subprocess call gone wrong
+        pass
+
+    if cputype:
+        # OK, found. Remove unwanted spaces:
+        cputype = " ".join(cputype.split())
+    else:
+        try:
+            # Not found, so let's fall back to platform()
+            cputype = platform.platform()
+        except:
+            # Can fail on special platforms (like Snapcraft or embedded)
+            pass
+
+    logging.debug("CPU model = %s", cputype)
+    return cputype
+
+
 def on_cleanup_list(filename: str, skip_nzb: bool = False) -> bool:
     """Return True if a filename matches the clean-up list"""
     cleanup_list = cfg.cleanup_list()
@@ -779,12 +869,13 @@ def format_time_string(seconds: float) -> str:
     return " ".join(completestr)
 
 
-def int_conv(value: Any) -> int:
-    """Safe conversion to int (can handle None)"""
+def int_conv(value: Any, default: Any = 0) -> int:
+    """Safe conversion to int (can handle None)
+    Returns 0 or requested default value"""
     try:
         return int(value)
     except:
-        return 0
+        return default
 
 
 def create_https_certificates(ssl_cert, ssl_key):
@@ -1015,19 +1106,6 @@ def ip_extract() -> List[str]:
     return ips
 
 
-def get_server_addrinfo(host: str, port: int) -> socket.getaddrinfo:
-    """Return getaddrinfo() based on user settings"""
-    try:
-        if cfg.ipv6_servers():
-            # Standard IPV4 or IPV6
-            return socket.getaddrinfo(host, port, socket.AF_UNSPEC, socket.SOCK_STREAM)
-        else:
-            # Only IPv4
-            return socket.getaddrinfo(host, port, socket.AF_INET, socket.SOCK_STREAM)
-    except:
-        return []
-
-
 def get_base_url(url: str) -> str:
     """Return only the true root domain for the favicon, so api.oznzb.com -> oznzb.com
     But also api.althub.co.za -> althub.co.za
@@ -1162,15 +1240,14 @@ def run_command(cmd: List[str], **kwargs):
     return txt
 
 
-def run_script(script):
-    """Run a user script (queue complete only)"""
-    script_path = make_script_path(script)
-    if script_path:
+def run_script(script: str):
+    """Run a user script"""
+    if script_path := make_script_path(script):
         try:
-            script_output = run_command([script_path])
-            logging.info("Output of queue-complete script %s: \n%s", script, script_output)
+            script_output = run_command([script_path], env=sabnzbd.newsunpack.create_env())
+            logging.info("Output of script %s: \n%s", script, script_output)
         except:
-            logging.info("Failed queue-complete script %s, Traceback: ", script, exc_info=True)
+            logging.info("Failed script %s, Traceback: ", script, exc_info=True)
 
 
 def set_socks5_proxy():
@@ -1200,33 +1277,6 @@ def set_https_verification(value):
     return prev
 
 
-def test_cert_checking():
-    """Test quality of certificate validation"""
-    # User disabled the test, assume proper SSL certificates
-    if not cfg.selftest_host():
-        return True
-
-    # Try a connection to our test-host
-    try:
-        ctx = ssl.create_default_context()
-        base_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        ssl_sock = ctx.wrap_socket(base_sock, server_hostname=cfg.selftest_host())
-        ssl_sock.settimeout(2.0)
-        ssl_sock.connect((cfg.selftest_host(), 443))
-        ssl_sock.close()
-        return True
-    except (socket.gaierror, socket.timeout):
-        # Non-SSL related error.
-        # We now assume that certificates work instead of forcing
-        # lower quality just because some (temporary) internet problem
-        logging.info("Could not determine system certificate validation quality due to connection problems")
-        return True
-    except:
-        # Seems something is still wrong
-        set_https_verification(False)
-    return False
-
-
 def request_repair():
     """Request a full repair on next restart"""
     path = os.path.join(cfg.admin_dir.get_path(), REPAIR_REQUEST)
@@ -1289,24 +1339,17 @@ def system_standby():
 
 
 def change_queue_complete_action(action: str, new: bool = True):
-    """Action or script to be performed once the queue has been completed
-    Scripts are prefixed with 'script_'
-    """
-    _action = None
-    _argument = None
+    """Action or script to be performed once the queue has been completed"""
+    function = None
     if new or cfg.queue_complete_pers():
-        if action.startswith("script_") and is_valid_script(action.replace("script_", "", 1)):
-            # all scripts are labeled script_xxx
-            _action = sabnzbd.misc.run_script
-            _argument = action.replace("script_", "", 1)
-        elif action == "shutdown_pc":
-            _action = system_shutdown
+        if action == "shutdown_pc":
+            function = system_shutdown
         elif action == "hibernate_pc":
-            _action = system_hibernate
+            function = system_hibernate
         elif action == "standby_pc":
-            _action = system_standby
+            function = system_standby
         elif action == "shutdown_program":
-            _action = sabnzbd.shutdown_program
+            function = sabnzbd.shutdown_program
         else:
             action = None
     else:
@@ -1317,8 +1360,7 @@ def change_queue_complete_action(action: str, new: bool = True):
         config.save_config()
 
     sabnzbd.QUEUECOMPLETE = action
-    sabnzbd.QUEUECOMPLETEACTION = _action
-    sabnzbd.QUEUECOMPLETEARG = _argument
+    sabnzbd.QUEUECOMPLETEACTION = function
 
 
 def keep_awake():
